<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:msxsl="urn:schemas-microsoft-com:xslt">

<!-- Entry point of the transform - start here -->
<xsl:template match="/">
	<xsl:variable name="resultsperday" select="APS_model_level_variables/APSFileHeader/ResultsPerDay"/>	<!-- samples per day - store this to pass in to various functions as its used in multiple locations -->

	<apsXML>
		<xsl:call-template name="GenerateHeaderInfo">
			<xsl:with-param name="resultsperday" select="$resultsperday"/>
		</xsl:call-template>
		<xsl:call-template name="GenerateResultsSection">
			<xsl:with-param name="resultsperday" select="$resultsperday"/>
		</xsl:call-template>
	</apsXML>
</xsl:template>

<!-- This section generates the HeaderInfo section of the destination XML file -->
<xsl:template name="GenerateHeaderInfo">
	<xsl:param name="resultsperday"/>
	<HeaderInfo>
		<SourceDescription>
			<xsl:value-of select="APS_model_level_variables/APSFileHeader/SourceDescription"/>
		</SourceDescription>
		<FileDescription>Building performance data, derived from XML</FileDescription>	<!-- FIXED for now -->
		<ClimateFile>
			<xsl:value-of select="APS_model_level_variables/APSFileHeader/ClimateFile"/>
		</ClimateFile>
		<Models></Models>
		<SimFlags></SimFlags>
		<SimPeriod presimdays = "0">	<!-- FIXED for now -->
			<Start>
				<xsl:attribute name="day">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/StartDay"/>				
				</xsl:attribute>
				<xsl:attribute name="month">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/StartMonth"/>				
				</xsl:attribute>			
				<xsl:attribute name="year">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/StartYear"/>				
				</xsl:attribute>			
			</Start>
			<End>
				<xsl:attribute name="day">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/EndDay"/>				
				</xsl:attribute>
				<xsl:attribute name="month">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/EndMonth"/>				
				</xsl:attribute>			
				<xsl:attribute name="year">
					<xsl:value-of select="APS_model_level_variables/APSFileHeader/EndYear"/>				
				</xsl:attribute>	
			</End>
		</SimPeriod>
		<OutputOptions>
			<xsl:attribute name="resultsperday">
				<xsl:value-of select="$resultsperday"/>				
			</xsl:attribute>
			<xsl:attribute name="plotdataoffsetseconds">
				<xsl:value-of select="APS_model_level_variables/APSFileHeader/Seconds"/>				
			</xsl:attribute>		
		</OutputOptions>
		<xsl:call-template name="GenerateVariablesSection"/>
	</HeaderInfo>
</xsl:template>
	
<!-- Helper funciton that removes the underscores from a string -->
<xsl:template name="RemoveAndReplaceUS">
	<xsl:param name="value"/>

	<xsl:variable name="firstchar" select="substring($value, 1, 1)"/>

	<xsl:if test="$firstchar = '_'">
	<xsl:variable name="removedfirstchar" select="substring($value, 2)"/>		<!-- strip the _ and recall this function -->
		<xsl:call-template name="RemoveAndReplaceUS">
			<xsl:with-param name="value" select="$removedfirstchar"/>
		</xsl:call-template>	
	</xsl:if>
	
	<xsl:if test="$firstchar != '_'">
		<xsl:value-of select="translate($value, '_', ' ')"/>
	</xsl:if>	
</xsl:template>

<!-- Extract the variable names etc. from the xml and reformat -->
<!-- Assumes the first EnergyValue node will be the same as all nodes in the source XML -->
<xsl:template name="GenerateVariablesSection">
	<xsl:variable name="units" select="APS_model_level_variables/APSFileHeader/Units"/>	
	
	<VariablesInOrder type = "BuildingEnergy" enum="131">
		<xsl:if test="APS_model_level_variables/Units">
			<xsl:for-each select="APS_model_level_variables/Units/child::*">
				<Variable>
					<xsl:variable name="name">
						<xsl:call-template name="RemoveAndReplaceUS">
							<xsl:with-param name="value" select="name()"/>	<!-- Format the name correctly -->
						</xsl:call-template>
					</xsl:variable>
					<xsl:attribute name="name">
						<xsl:value-of select="$name"/>
					</xsl:attribute>
					<xsl:attribute name="level">
						<xsl:value-of select="'m'"/>	<!-- FIXED for now -->
					</xsl:attribute>
					<xsl:attribute name="units">
						<xsl:value-of select="."/>
					</xsl:attribute>
				</Variable>
			</xsl:for-each>	
		</xsl:if>
		<xsl:if test="not(APS_model_level_variables/Units)">		
			<xsl:for-each select="APS_model_level_variables/EnergyValue[1]/child::*">	<!-- Loop round first node -->		
				<Variable>
					<xsl:variable name="name">
						<xsl:call-template name="RemoveAndReplaceUS">
							<xsl:with-param name="value" select="name()"/>		<!-- Format the name correctly -->
						</xsl:call-template>	
					</xsl:variable>			
					<xsl:attribute name="name">
						<xsl:value-of select="$name"/>		
					</xsl:attribute>
					<xsl:attribute name="level">
						<xsl:value-of select="'m'"/>		<!-- FIXED for now -->
					</xsl:attribute>
					<xsl:attribute name="units">
						<xsl:value-of select="$units"/>
					</xsl:attribute>
				</Variable>
			</xsl:for-each>
		</xsl:if>
	</VariablesInOrder>
</xsl:template>

<!-- This section generates the Results section of the destination XML file -->
<xsl:template name="GenerateResultsSection">
	<xsl:param name="resultsperday"/>
	<Results>	
		<xsl:variable name="energynodes" select="APS_model_level_variables/EnergyValue"/>
		<!-- Use recursion to loop round all of the EnergyValue Nodes -->
		<xsl:call-template name="ProcessEnergyNode">
			<xsl:with-param name="nodes" select="$energynodes"/>
			<xsl:with-param name="nodecount" select="count($energynodes)"/>
			<xsl:with-param name="samplecount" select="$resultsperday"/>
			<xsl:with-param name="currentcount" select="0"/>
		</xsl:call-template>	
	</Results>
</xsl:template>

	
<!-- Process the EnergyValue nodes in chunks of samplecount sections (96 in our original data set) -->
<xsl:template name="ProcessEnergyNode">
	<xsl:param name="nodes"/>			<!-- Complete set of EnergyValue nodes (in original sample this was 35040 nodes) -->
	<xsl:param name="nodecount"/>		<!-- Count of the EnergyValue nodes (in original sample this was 35040) -->
	<xsl:param name="samplecount"/>		<!-- Count of the results per day (in original sample this was 96) -->
	<xsl:param name="currentcount"/>	<!-- Start position of node set to process (we process the EnergyValue node in samplecount chunks) -->

	<xsl:if test="$nodecount >= $samplecount">	<!-- Only proceed if we have more than samplecount items to process -->
		
		<xsl:call-template name="ProcessEnergyNodeChunk">
			<!-- Pass parameter with "samplecount" nodes to process -->
			<xsl:with-param name="daynodes">
				<xsl:for-each select="$nodes[position()&gt;$currentcount and position()&lt;=($currentcount+$samplecount)]">
					<xsl:copy-of select="."/>
				</xsl:for-each>
			</xsl:with-param>
			<xsl:with-param name="variablecount" select="count($nodes[1]/*)"/>
			<xsl:with-param name="daynumber" select="($currentcount div $samplecount) + 1"/>
		</xsl:call-template>
		
		<!-- Process the next EnergyValue node chunk -->
		<xsl:call-template name="ProcessEnergyNode">
			<xsl:with-param name="nodes" select="$nodes"/>							<!-- No change -->
			<xsl:with-param name="nodecount" select="$nodecount - $samplecount"/>	<!-- we have processed samplecount items so subtract from overall total -->
			<xsl:with-param name="samplecount" select="$samplecount"/>				<!-- No change -->
			<xsl:with-param name="currentcount" select="$currentcount + $samplecount"/>	<!-- Start position of the next node set to process -->
		</xsl:call-template>
	</xsl:if>
</xsl:template>	
	
<!-- We have a chunk of EnergyValue nodes (e.g. 96) - now process this  -->	
<xsl:template name="ProcessEnergyNodeChunk">
	<xsl:param name="daynodes"/>		<!-- Chunk of EnergyValue nodes - a days worth of samples for ALL variables--> 
	<xsl:param name="variablecount"/>	<!-- Number of variables per EnergyValue node -->
	<xsl:param name="daynumber"/>		<!-- Day number 1,2,3, .. ,365 - sanity check -->
	<Day>
		<xsl:attribute name="date">
			<xsl:value-of select="$daynumber"/>
		</xsl:attribute>
		<DataSection type = "BuildingEnergy" enum = "132">
			<DataValues enum = "133">
				<xsl:call-template name="ProcessEnergyValueVariables">
					<xsl:with-param name="daynodes" select="$daynodes"/>			<!-- Copy of original chunk -->
					<xsl:with-param name="variablecount" select="$variablecount"/>	<!-- Copy of number of varibles in each EnergyValue node -->
					<xsl:with-param name="currentcount" select="1"/>				<!-- to keep track of how many variables we have processed -->
				</xsl:call-template>
				</DataValues>
		</DataSection>
	</Day>
</xsl:template>
	
<!-- Generate the result string -->
<xsl:template name="ProcessEnergyValueVariables">
	<xsl:param name="daynodes"/>
	<xsl:param name="variablecount"/>
	<xsl:param name="currentcount"/>

	<xsl:if test="$variablecount >= $currentcount">

		<!-- Get the name of the node to store in the results - sanity check -->
		<xsl:variable name="resultname">
			<xsl:call-template name="RemoveAndReplaceUS">
				<xsl:with-param name="value" select="local-name(msxsl:node-set($daynodes)/*/*[$currentcount])"/>	<!-- Format the name correctly -->
			</xsl:call-template>
		</xsl:variable>
				
		<!-- Convert all of the nodes into a space delimited string -->
		<xsl:variable name="results">
			<!-- this generates the large string with all of the data -->
			<xsl:for-each select="msxsl:node-set($daynodes)/child::*">
				<xsl:value-of select="concat(./*[$currentcount], ' ')"/>															
			</xsl:for-each>			
		</xsl:variable>
		
		<!-- Now store this string in a Variable node -->
		<Variable>
			<xsl:attribute name="name">
				<xsl:value-of select="$resultname"/>
			</xsl:attribute>
			<xsl:attribute name="results">
				<xsl:value-of select="$results"/>
			</xsl:attribute>	
		</Variable>
		
		<!-- Recurse to process the next variable in the EnergyValue Node -->
		<xsl:call-template name="ProcessEnergyValueVariables">
			<xsl:with-param name="daynodes"  select="$daynodes"/>
			<xsl:with-param name="variablecount" select="$variablecount"/>
			<xsl:with-param name="currentcount" select="$currentcount + 1"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
